/**
  *****************************************************************************
  * @file    {file}
  * @author  {author}
  * @date    {date}
  * @brief   Version definition.
  * @note    Generated automatically. DO NOT EDIT.
  *****************************************************************************
  */
#ifndef {guard}
#define {guard}

#define VERSION_MAJOR   {major}
#define VERSION_MINOR   {minor}
#define VERSION_BUILD   {build}
#define VERSION_STRING  "{version}"

#endif /* {guard} */
